{
=============================================================================
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"  |
| AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE    |
| IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   |
| ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR  |
| ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL       |
| DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR   |
| SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER   |
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT           |
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY    |
| OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  |
| DAMAGE.                                                                      |
=============================================================================
   Author:  D.J.C. Blijker.
   Date:    13-4-2003
   version: 1.0
   e-mail:  c.blijker@wxs.nl
=============================================================================
  This unit will work in Delphi as well in Kylix!!!
=============================================================================

 This unit is part of the service broker application.
 It supplies the functionality to act as an http-server..
 A tcp-server is created which is listening on port 2002.
 To connect to this service type:
 http://<server-name>:2002
 for instance: http://localhost:2002
==============================================================================
History:

  version: 1.0  Initial release.
==============================================================================
}
unit ServiceBroker_HttpLayer;

interface

uses
{$IFDEF LINUX}
  libc, kernelioctl,
{$ELSE}
  Windows, WinSock,
{$ENDIF}
  SysUtils, ServiceBroker_LogicLayer, ServiceBroker_Shared, HxTcpIpcUdp,
  HxDataExchange;

type
  TErrorType = ( etOk, etMissingField, etNoIp );

  THxHttpServiceBroker = class
  private
    tcp: THxTcpServer;
    sb: THxServiceBroker;
    bOwnerOfDe: boolean;
    de: THxDataExchange;
    procedure HandleDataAvailable( ptr: pointer; iSize, iDescr: integer );
    procedure HandleNotify( sMess: string );
    procedure SendHtmlPage( iDescr: integer );
    function  ProtocolToText( eProtocol: THxProtocol ): string;
    function  AddNewService( s: string ): TErrorType;
    procedure SendNoIpWarning(iDescr: integer);
    procedure SendMissingField(iDescr: integer);
  public
    constructor Create( sbServBroker: THxServiceBroker = nil );
    destructor Destroy; override;
  end;


implementation


{ THxHttpServiceBroker }

constructor THxHttpServiceBroker.Create( sbServBroker: THxServiceBroker );
begin
  if sbServBroker = nil then
  begin
    sb := THxServiceBroker.Create;
    bOwnerOfDe := True;
  end
  else
  begin
    sb := sbServBroker;
    bOwnerOfDe := False;
  end;

// Listen on port 2002 for http-requests..
  tcp := THxTcpServer.Create;
  tcp.Port := '2002';
  tcp.OnDataAvailable := HandleDataAvailable;
  tcp.OnNotify := HandleNotify;
  tcp.Listen;

  de := THxDataExchange.Create;
end;

destructor THxHttpServiceBroker.Destroy;
begin
  if bOwnerOfDe then
    sb.Free;
  tcp.Free;
  de.Free;

  inherited;
end;

procedure THxHttpServiceBroker.HandleDataAvailable( ptr: pointer; iSize,
                                                    iDescr: integer );
var
  s: string;
  eRes: TErrorType;
begin
  SetLength( s, iSize );
{$IFDEF LINUX}
  memcpy( pointer( s ), ptr, iSize );
{$ELSE}
  CopyMemory( pointer( s ), ptr, iSize );
{$ENDIF}
  try
    if Pos('HTTP/1.1', s ) > 0 then
    begin
      if Pos('GET ', s ) > 0 then
        SendHtmlPage( iDescr )
      else if Pos('POST /AddNewService ', s )> 0 then
      begin
        eRes := AddNewService( s );
        case eRes of
          etOk: SendHtmlPage( iDescr );
          etNoIp: SendNoIpWarning( iDescr );
          etMissingField: SendMissingField( iDescr );
        else
          raise Exception.Create('unknown service result found');
        end;
      end;
    end;
  finally
    tcp.DisconnectClient( iDescr );
  end;
end;

function THxHttpServiceBroker.AddNewService( s: string ): TErrorType;
var
  sServiceName, sServiceAddress, sServicePort: string;
  iP1, iP2: integer;
  eProtocol: THxProtocol;
begin
  iP1 := Pos('ServiceName=', s );
  iP2 := Pos('&', s );
  sServiceName := Copy( s, iP1+12, iP2 - iP1 - 12 );

  s := Copy( s, iP2+1, 9999 );
  iP1 := Pos('ServiceAddress=', s );
  iP2 := Pos('&', s );
  sServiceAddress := Copy( s, iP1+15, iP2 - iP1 - 15 );

  s := Copy( s, iP2+1, 9999 );
  iP1 := Pos('ServicePort=', s );
  iP2 := Pos('&', s );
  sServicePort := Copy( s, iP1+12, iP2 - iP1 - 12 );

  if( sServiceName = '')or( sServiceAddress = '' )or( sServicePort = '' )then
  begin
    result := etMissingField;
    exit;
  end;

  sServiceAddress := ValidateIpAddress( sServiceAddress );
  if sServiceAddress = '' then
  begin
    result := etNoIp;
    exit;
  end;

  s := Copy( s, iP2+1, 9999 );
  if Pos('UDP', s )<> 0 then
    eProtocol := pcUdp
  else if Pos('TCP', s )<> 0 then
    eProtocol := pcTcp
  else
    eProtocol := pcIpc;

  sb.AddService( sServiceName, sServiceAddress, sServicePort, eProtocol );

  result := etOk;
end;

procedure THxHttpServiceBroker.HandleNotify(sMess: string);
begin
end;

function THxHttpServiceBroker.ProtocolToText(eProtocol: THxProtocol): string;
begin
  case eProtocol of
    pcTcp: result := 'TCP/IP';
    pcIpc: result := 'IPC';
    pcUdp: result := 'UDP';
  else
    result := 'Unknown';
  end;
end;

procedure THxHttpServiceBroker.SendNoIpWarning( iDescr: integer );
var
  sHtml, sHeader: string;
begin
  sHtml := '<HTML>'+
           '<BODY>'+
           '<CENTER><H2>'+
           'Not a valid service address'+
           '</H2>'+
           '</BODY></HTML>'+#13+#10;


  sHeader := 'HTTP/1.1 200 OK'+#13+#10+
             'Accept-Ranges: bytes'+#13+#10+
             'Content-Length: '+IntToStr( Length( sHtml ))+#13+#10+
             'Connection: close '+#13+#10+
             'Content-Type: text/html'+#13+#10+
             ''+#13+#10+
             ''+#13+#10;

  tcp.SendData( pchar( sHeader ), Length( sHeader ), iDescr );
  tcp.SendData( pchar( sHtml ), Length( sHtml ), iDescr );
end;

procedure THxHttpServiceBroker.SendMissingField( iDescr: integer );
var
  sHtml, sHeader: string;
begin
  sHtml := '<HTML>'+
           '<BODY>'+
           '<CENTER><H2>'+
           'Not all fields are filled in'+
           '</H2>'+
           '</BODY></HTML>'+#13+#10;


  sHeader := 'HTTP/1.1 200 OK'+#13+#10+
             'Accept-Ranges: bytes'+#13+#10+
             'Content-Length: '+IntToStr( Length( sHtml ))+#13+#10+
             'Connection: close '+#13+#10+
             'Content-Type: text/html'+#13+#10+
             ''+#13+#10+
             ''+#13+#10;

  tcp.SendData( pchar( sHeader ), Length( sHeader ), iDescr );
  tcp.SendData( pchar( sHtml ), Length( sHtml ), iDescr );
end;

procedure THxHttpServiceBroker.SendHtmlPage(iDescr: integer);
var
  sHeader, sHtml: string;
  p: PHxServiceLong;
  i, iNOS: integer;
begin
  sHtml := '<HTML>'+
           '<BODY>'+
           '<CENTER><H2>'+
           'Service Broker'+
           '</H2>'+
           '<BR><HR><BR>'+
           '<TABLE BORDER="1"><BR>';

  iNOS := sb.NrOfServices;
  sHtml := sHtml + '<TR><TD COLSPAN="4" ALIGN="CENTER"><H3>'+
           'List of known services:</H3></TD></TR>'+
           '<TR><TD>ServiceName: </TD><TD>ServiceAddress: </TD>'+
           '<TD>ServicePort: </TD><TD> ServiceProtocol: </TD></TR>';
  for i := 0 to iNOS -1 do
  begin
    p := sb.ServiceInfo[i];
    sHtml := sHtml + '<TR><TD>'+p^.sServiceName+'</TD>'+
             '<TD>'+ p^.sAddress + '</TD>'+
             '<TD>'+ p^.sPort+'</TD>'+
             '<TD>'+ ProtocolToText( p^.eProtocolType )+'</TD></TR>';
  end;

  sHtml := sHtml+'</TABLE>'+#13+#10;

  sHtml := sHtml + '<BR><BR><HR><BR><BR>'+
           '<H3>Add a new service</H3>'+
           '<TABLE BORDER="0"><BR>';

  sHtml := sHtml +'<FORM ACTION="AddNewService" METHOD=POST>'+
           '<TR><TD>'+
           'Service name: '+
           '<INPUT TYPE=TEXT NAME="ServiceName"></TD>'+
           '<TD>Service IP-Address: '+
           '<INPUT TYPE=TEXT NAME="ServiceAddress"></TD>'+
           '<TD>Service Port: '+
           '<INPUT TYPE=TEXT NAME="ServicePort"></TD></TR>'+
           '<TR><TD COLSPAN="3">Protocol: </TD></TR>'+
           '<TR><TD><INPUT TYPE=RADIO NAME="TCP" CHECKED>Tcp</TD>'+
           '<TD><INPUT TYPE=RADIO NAME="IPC">Ipc</TD>'+
           '<TD><INPUT TYPE=RADIO NAME="UDP">Udp</TD></TR>'+
           '</TD></TR></TABLE><BR>'+
           '<INPUT TYPE=SUBMIT VALUE="Submit">'+
           '</FORM></CENTER></BODY></HTML>'+#13+#10;


  sHeader := 'HTTP/1.1 200 OK'+#13+#10+
             'Accept-Ranges: bytes'+#13+#10+
             'Content-Length: '+IntToStr( Length( sHtml ))+#13+#10+
             'Connection: close '+#13+#10+
             'Content-Type: text/html'+#13+#10+
             ''+#13+#10+
             ''+#13+#10;

  tcp.SendData( pchar( sHeader ), Length( sHeader ), iDescr );
  tcp.SendData( pchar( sHtml ), Length( sHtml ), iDescr );
end;

end.
